import boto3

from .smsabstract import SMSApi
from ..models import SMSNotificationType


class SMSAPISNSAmazon(SMSApi):

    def __init__(self, key_id, access_key, region_name, **kwargs):
        super(SMSAPISNSAmazon, self).__init__(**kwargs)
        self.api = boto3.client(
            'sns',
            aws_access_key_id=key_id,
            aws_secret_access_key=access_key,
            region_name=region_name,
        )

    def bulk_send(self, phone_numbers: [str], message: str, sms_type: SMSNotificationType):
        # TODO: handle bulk options
        for phone_number in phone_numbers:
            self.send(phone_number, message, sms_type)

    def send(self, phone_number: str, message: str, sms_type: SMSNotificationType):
        sms_type_string = 'Promotional' if sms_type == SMSNotificationType.PROMOTIONAL else 'Transactional'
        response = self.api.publish(
            PhoneNumber=phone_number,
            Message=message,
            MessageAttributes={
                'AWS.SNS.SMS.SMSType': {
                    'DataType': 'String',
                    'StringValue': sms_type_string
                }
            }
        )
        # TODO: handle response



from azmessaging import default_settings as settings
from azmessaging.utils import get_continent, import_class
from .bases import Reader
from ..sms import SMSApi


class DefaultReader(Reader):

    def read(self, channel: str, identifier: str) -> dict:
        """

        :param channel:
        :param identifier:
        :return:
        base on channel type for example for sms:
        'SMS': {
            'SNS': {
                'ACCESS_KEY_ID': os.environ.get('SNS_ACCESS_KEY_ID', None),
                'SECRET_ACCESS_KEY': os.environ.get('SNS_SECRET_ACCESS_KEY', None),
            },
            'TWILIO': {
                'ACCOUNT_SID': os.environ.get('TWILIO_ACCOUNT_SID', None),
                'AUTH_TOKEN': os.environ.get('TWILIO_AUTH_TOKEN', None),
                'SENDER': os.environ.get('TWILIO_SENDER', None),
            }
        }
        """
        return getattr(settings, 'MESSAGING', {}).get(channel, {})

    def get_sms_sender(self, identifier: str, country_code: str) -> SMSApi:
        continent = get_continent(country_code).upper()
        for sp_name in settings.SMS_CONFIG['PRIORITY_SERVICE_PROVIDER']:
            kwargs = {}
            sp = settings.SMS_CONFIG['SERVICE_PROVIDER'][sp_name]
            kwargs.update(sp)
            del kwargs['ROUTING']
            del kwargs['CLASS']
            for r in sp['ROUTING']:
                if continent in r['continents'] or country_code in r['countries']:
                    r_copy = r.copy()
                    del r_copy['continents']
                    del r_copy['countries']
                    kwargs.update(r_copy)
                    klass = import_class(sp['CLASS'])
                    return klass(**kwargs)



from phonenumbers import region_code_for_country_code, parse as phone_number_parse
from azmessaging.models import SMSNotificationType
from .base import BaseNotificationChannel


class SMSNotificationChannel(BaseNotificationChannel):
    """
    structure: {"country_code": []}
    for example:
    {"US": ["+15124467"], "UK": []}
    """
    _receivers = {}
    sms_type = SMSNotificationType.PROMOTIONAL

    def __init__(self, sms_type, **kwargs):
        super(SMSNotificationChannel, self).__init__(**kwargs)
        self.sms_type = sms_type

    def _send_msg(self, message, phone_numbers, country_code):
        sender = self.reader.get_sms_sender(self.identifier, country_code=country_code)
        sender.bulk_send(phone_numbers, message, self.sms_type)

    def set_receivers(self, phone_numbers: list):
        self._receivers = {}
        for item in phone_numbers:
            country_code = region_code_for_country_code(phone_number_parse(item).country_code)
            if country_code not in self._receivers:
                self._receivers[country_code] = []
            self._receivers[country_code].append(item)

    def get_receivers(self):
        return self._receivers

    def get_message(self):
        return self.raw_message

    def notify(self):
        """Sends the notification."""
        for country_code in self.get_receivers():
            self._send_msg(self.get_message(), phone_numbers=self.get_receivers()[country_code], country_code=country_code)



import abc

import six

from azmessaging import default_settings as settings
from azmessaging.utils import import_class


@six.add_metaclass(abc.ABCMeta)
class BaseNotificationChannel:
    identifier = None
    """Base channel for sending notifications."""

    def __init__(self, identifier: str, message: str, **kwargs):
        self.identifier = identifier
        self.raw_message = message
        self.reader = import_class(settings.SETTING_VALUE_READER_CLASS)()

    @abc.abstractmethod
    def get_message(self):
        """get message."""
        pass

    @abc.abstractmethod
    def notify(self):
        """Sends the notification."""
        pass



import importlib

import pytz


def import_class(path):
    package, attr = path.rsplit('.', 1)
    klass = getattr(importlib.import_module(package), attr)
    return klass


def get_continent(country_code):
    continent = pytz.country_timezones[country_code][0].split('/')[0]
    return continent



import abc

import pytz
import six

from azmessaging import default_settings as settings
from azmessaging.sms import SMSApi


@six.add_metaclass(abc.ABCMeta)
class Reader:

    @abc.abstractmethod
    def read(self, channel: str, identifier: str) -> dict:
        """

        :param channel:
        :param identifier:
        :return:
        base on channel type for example for sms:
        'SMS': {
            'SNS': {
                'ACCESS_KEY_ID': os.environ.get('SNS_ACCESS_KEY_ID', None),
                'SECRET_ACCESS_KEY': os.environ.get('SNS_SECRET_ACCESS_KEY', None),
            },
            'TWILIO': {
                'ACCOUNT_SID': os.environ.get('TWILIO_ACCOUNT_SID', None),
                'AUTH_TOKEN': os.environ.get('TWILIO_AUTH_TOKEN', None),
                'SENDER': os.environ.get('TWILIO_SENDER', None),
            }
        }
        """
        pass

    def klass(self, channel: str, identifier: str) -> dict:
        return settings.CHANNEL_CLASS[channel.upper()]

    @abc.abstractmethod
    def get_sms_sender(self, identifier: str, country_code: str) -> SMSApi:
        pass
